package eu.jonahbauer.json.query;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.parser.JQParser;
import eu.jonahbauer.json.query.parser.ast.JQExpression;
import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.IOException;
import java.io.UncheckedIOException;
import java.util.stream.Stream;

@RequiredArgsConstructor(access = AccessLevel.PRIVATE)
public class JsonQuery {
    private final @NotNull JQExpression expression;

    public static @NotNull JsonQuery parse(@NotNull String query) {
        try {
            var parser = new JQParser(query);
            var programm = parser.parseTopLevel();
            if (programm.expression() == null) throw new IllegalArgumentException();
            return new JsonQuery(programm.expression());
        } catch (IOException ex) {
            throw new UncheckedIOException(ex);
        }
    }

    public @NotNull Stream<@NotNull JsonValue> run(@Nullable JsonValue value) {
        var context = new JQExpression.Context(value);
        return expression.evaluate(context);
    }
}
