package eu.jonahbauer.json.query.impl;

import org.jetbrains.annotations.NotNull;

import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

final class ConcatGenerator<T> implements Generator<T> {
    private final @NotNull Queue<@NotNull Generator<? extends T>> generators;

    public ConcatGenerator(@NotNull List<@NotNull Generator<? extends T>> generators) {
        this.generators = new LinkedList<>(generators);
    }

    @Override
    public T next() throws EndOfStreamException {
        while (!generators.isEmpty()) {
            try {
                var current = generators.peek();
                return current.next();
            } catch (EndOfStreamException ex) {
                generators.remove();
            }
        }
        throw new EndOfStreamException();
    }
}
