package eu.jonahbauer.json.query.impl;

import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.function.Function;

public interface Generator<T> {
    T next() throws EndOfStreamException;

    default <S> @NotNull Generator<S> map(@NotNull Function<? super T, ? extends S> function) {
        return new MappingGenerator<>(this, function);
    }

    default <S> @NotNull Generator<S> flatMap(@NotNull Function<? super T, ? extends Generator<? extends S>> function) {
        return new FlatMappingGenerator<>(this, function);
    }

    @SuppressWarnings("unchecked")
    static <T> @NotNull Generator<T> empty() {
        return (Generator<T>) EmptyGenerator.INSTANCE;
    }

    static <T> @NotNull Generator<T> concat(@NotNull Generator<? extends T> first, @NotNull Generator<? extends T> second) {
        return new ConcatGenerator<>(List.of(first, second));
    }

    static <T> @NotNull Generator<T> of(T value) {
        return new SingletonGenerator<>(value);
    }

    class EndOfStreamException extends RuntimeException {}
}
