package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import eu.jonahbauer.json.query.JsonMath;
import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.BinaryOperator;
import java.util.function.Supplier;
import java.util.stream.Gatherer;
import java.util.stream.Stream;

@SuppressWarnings("preview")
public record JQAlternativeExpression(@NotNull JQExpression first, @NotNull JQExpression second) implements JQExpression {
    public JQAlternativeExpression {
        Objects.requireNonNull(first);
        Objects.requireNonNull(second);
    }

    @Override
    public @NotNull Stream<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return first.evaluate(context).gather(new AlternativeGatherer(second, context));
    }

    @Override
    public boolean isConstant() {
        return first.isConstant() && second.isConstant();
    }

    @Override
    public @NotNull String toString() {
        return first + " // " + second;
    }

    @NoArgsConstructor
    @AllArgsConstructor
    private static class State {
        private boolean empty = true;
    }

    private record AlternativeGatherer(
            @NotNull JQExpression expression,
            @NotNull Context context
    ) implements Gatherer<JsonValue, State, JsonValue> {

        @Override
        public @NotNull Supplier<State> initializer() {
            return State::new;
        }

        @Override
        public @NotNull Integrator<State, JsonValue, JsonValue> integrator() {
            return Integrator.ofGreedy((state, element, downstream) -> {
                if (JsonMath.isTruthy(element)) {
                    state.empty = false;
                    return downstream.push(element);
                }
                return true;
            });
        }

        @Override
        public @NotNull BinaryOperator<State> combiner() {
            return (state1, state2) -> new State(state1.empty && state2.empty);
        }

        @Override
        public @NotNull BiConsumer<State, Downstream<? super JsonValue>> finisher() {
            return (state, downstream) -> {
                if (!state.empty) return;
                var it = expression.evaluate(context).iterator();
                while (it.hasNext() && downstream.push(it.next()));
            };
        }
    }
}
