package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonValue;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;
import java.util.function.Supplier;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

public record JQCommaExpression(@NotNull JQExpression first, @NotNull JQExpression second) implements JQExpression {
    public JQCommaExpression {
        Objects.requireNonNull(first);
        Objects.requireNonNull(second);
    }

    @Override
    public @NotNull Stream<@Nullable JsonValue> evaluate(@NotNull Context context) {
        var it = new Iterator<JsonValue>() {
            private Iterator<JsonValue> delegate;
            private Queue<Supplier<Iterator<JsonValue>>> queue = new LinkedList<>(List.of(
                    () -> first.evaluate(context).iterator(),
                    () -> second.evaluate(context).iterator()
            ));

            @Override
            public boolean hasNext() {
                if (delegate == null) delegate = queue.remove().get();
                while (!delegate.hasNext() && !queue.isEmpty()) {
                    delegate = queue.remove().get();
                }
                return delegate.hasNext();
            }

            @Override
            public JsonValue next() {
                if (delegate == null) delegate = queue.remove().get();
                while (!delegate.hasNext() && !queue.isEmpty()) {
                    delegate = queue.remove().get();
                }
                return delegate.next();
            }
        };
        var spliterator = Spliterators.spliteratorUnknownSize(it, Spliterator.ORDERED);
        return StreamSupport.stream(spliterator, false);
    }

    @Override
    public boolean isConstant() {
        return first.isConstant() && second.isConstant();
    }

    @Override
    public @NotNull String toString() {
        return first + ", " + second;
    }
}
