package eu.jonahbauer.json.query.parser.ast;

import eu.jonahbauer.json.JsonArray;
import eu.jonahbauer.json.JsonObject;
import eu.jonahbauer.json.JsonValue;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.stream.Stream;

public record JQRecursionExpression() implements JQExpression {
    @Override
    public @NotNull Stream<@Nullable JsonValue> evaluate(@NotNull Context context) {
        return recurse(context.root());
    }

    private @NotNull Stream<@Nullable JsonValue> recurse(@Nullable JsonValue value) {
        return switch (value) {
            case JsonArray array -> Stream.concat(Stream.of(array), array.stream().flatMap(this::recurse));
            case JsonObject object -> Stream.concat(Stream.of(object), object.values().stream().flatMap(this::recurse));
            case null, default -> Stream.of(value);
        };
    }

    @Override
    public boolean isConstant() {
        return false;
    }
}
